package sbservermonitor

import (
	"fmt"
	"log"
	"os"

	"git.ar21.de/yolokube/sb-server-monitor/hrobot"
	"github.com/prometheus/client_golang/prometheus"
)

type Collector struct {
	exitOnRequestError bool
	lastError          error
	requestErrorCount  int
	Data               []*hrobot.SbServer
}

func NewExporter(exitOnRequestError bool) *Collector {
	return &Collector{
		exitOnRequestError: exitOnRequestError,
		lastError:          nil,
		requestErrorCount:  0,
	}
}

func (c *Collector) Describe(ch chan<- *prometheus.Desc) {
	ch <- metricServerCount
	ch <- metricServers
}

func (c *Collector) Collect(ch chan<- prometheus.Metric) {
	if len(c.Data) == 0 {
		log.Print("no data existing")
		if c.exitOnRequestError {
			os.Exit(1)
		}
	}

	c.collectServersMetric(ch, c.Data)
	c.collectServerCountMetric(ch, c.Data[0].Category, len(c.Data))
	c.collectServerCountPerPriceMetric(ch, c.Data)
	c.collectServerCountPerDatacenterMetric(ch, c.Data)
}

func (c *Collector) collectServerCountMetric(ch chan<- prometheus.Metric, category string, value int) {
	ch <- prometheus.MustNewConstMetric(
		metricServerCount,
		prometheus.GaugeValue,
		float64(value),
		category,
	)
}

func (c *Collector) collectServerCountPerPriceMetric(ch chan<- prometheus.Metric, servers []*hrobot.SbServer) {
	serverCache := make(map[int64]int)
	for _, server := range servers {
		serverCache[server.PriceInformation.Price] += 1
	}

	for price, amount := range serverCache {
		ch <- prometheus.MustNewConstMetric(
			metricServerCountPerPrice,
			prometheus.GaugeValue,
			float64(amount),
			fmt.Sprintf("%d €", price),
		)
	}
}

func (c *Collector) collectServerCountPerDatacenterMetric(ch chan<- prometheus.Metric, servers []*hrobot.SbServer) {
	serverCache := make(map[string]int)
	for _, server := range servers {
		serverCache[server.Datacenter] += 1
	}

	for datacenter, amount := range serverCache {
		ch <- prometheus.MustNewConstMetric(
			metricServerCountPerDatacenter,
			prometheus.GaugeValue,
			float64(amount),
			datacenter,
		)
	}
}

func (c *Collector) collectServersMetric(ch chan<- prometheus.Metric, servers []*hrobot.SbServer) {
	for _, server := range servers {
		ch <- prometheus.MustNewConstMetric(
			metricServers,
			prometheus.GaugeValue,
			float64(server.PriceInformation.Price),
			fmt.Sprint(server.ID),
			server.Name,
			server.Category,
			server.Hardware.CPU,
			server.Datacenter,
			fmt.Sprint(server.Information),
			fmt.Sprint(server.PriceInformation.Price),
			fmt.Sprint(server.PriceInformation.FixedPrice),
			fmt.Sprint(server.PriceInformation.NextReduceTimestamp),
		)
	}
}
