package main

import (
	"context"
	"log"
	"net/http"
	"os"
	"os/signal"
	"syscall"
	"time"

	sbservermonitor "git.ar21.de/yolokube/sb-server-monitor/collector/sb-server-monitor"
	"git.ar21.de/yolokube/sb-server-monitor/hrobot"
	"github.com/alecthomas/kingpin/v2"
	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promhttp"
)

type application struct {
	collector *sbservermonitor.Collector
	server    *http.Server
	config    *appSettings
}

func (app *application) refreshWorker(ctx context.Context, done chan<- interface{}) {
	sleepCounter := int(app.config.scrapeInterval)
	client := hrobot.NewClient()
	for {
		select {
		case <-ctx.Done():
			done <- nil
			return
		default:
			if sleepCounter/int(app.config.scrapeInterval) == 0 {
				break
			}

			log.Print("Fetching servers")
			sleepCounter = 0
			servers, err := client.SbServer.List(context.Background())
			if err != nil {
				log.Println(err)
				break
			}
			log.Printf("found %d servers", len(servers))

			app.collector.Data = servers
		}
		sleepCounter++
		time.Sleep(time.Second)
	}
}

func (app *application) webListener() {
	log.Printf("listening on %s", app.config.metricsAddress)

	http.Handle("/", http.RedirectHandler("/metrics", http.StatusPermanentRedirect))
	http.Handle("/metrics", promhttp.Handler())

	if err := app.server.ListenAndServe(); err != nil {
		log.Println(err)
	}
}

var app = application{}

type appSettings struct {
	metricsAddress     string
	exitOnRequestError bool
	scrapeInterval     int
}

func init() {
	var settings appSettings
	config := kingpin.New(os.Args[0], "sb-server-monitor")
	config.Flag("web.listen-address", "Address to use for the metrics server").Envar("SBSERVERMONITOR_WEB_LISTEN_ADDRESS").Default(":9192").StringVar(&settings.metricsAddress)
	config.Flag("collector.scrape-interval", "Interval for the data collection from the Hetzner Server Auction API (in seconds)").Envar("SBSERVERMONITOR_SCRAPE_INTERVAL").Default("30").IntVar(&settings.scrapeInterval)
	config.Flag("collector.exit-on-request-error", "When set to true the exporter will immediately exit on a request error").Envar("SBSERVERMONITOR_EXIT_ON_REQUEST_ERROR").Default("false").BoolVar(&settings.exitOnRequestError)
	kingpin.MustParse(config.Parse(os.Args[1:]))

	app.config = &settings

	app.collector = sbservermonitor.NewExporter(app.config.exitOnRequestError)
	prometheus.MustRegister(app.collector)

	app.server = &http.Server{
		Handler: nil,
		Addr:    app.config.metricsAddress,
	}
}

func main() {
	signals := make(chan os.Signal, 1)
	signal.Notify(signals, syscall.SIGTERM)
	signal.Notify(signals, syscall.SIGINT)

	ctx, cancelWorkers := context.WithCancel(context.Background())
	done := make(chan interface{})

	go app.webListener()

	log.Printf("start worker")
	go app.refreshWorker(ctx, done)

	<-signals

	cancelWorkers()

	if err := app.server.Shutdown(context.Background()); err != nil {
		log.Println(err)
	}

	log.Println("sb-server-monitor exporter stopped")
}
